/* AVI parser implementation */

/* Application inclusions */
#include "FT_Buffer.h"
#include "FT_MQueue.h"
#include "FT_AVIParser.h"
#include "FT_Platform.h"
/* AVI parser design */
/*
 * heres the general layout of an AVI riff file (new format)
 *
 * RIFF (3F??????) AVI       <- not more than 1 GB in size
 *     LIST (size) hdrl
 *         avih (0038)
 *         LIST (size) strl
 *             strh (0038)
 *             strf (????)
 *             indx (3ff8)   <- size may vary, should be sector sized
 *         LIST (size) strl
 *             strh (0038)
 *             strf (????)
 *             indx (3ff8)   <- size may vary, should be sector sized
 *         LIST (size) odml
 *             dmlh (????)
 *         JUNK (size)       <- fill to align to sector - 12
 *     LIST (7f??????) movi  <- aligned on sector - 12
 *         00dc (size)       <- sector aligned
 *         01wb (size)       <- sector aligned
 *         ix00 (size)       <- sector aligned
 *     idx1 (00??????)       <- sector aligned
 * RIFF (7F??????) AVIX
 *     JUNK (size)           <- fill to align to sector -12
 *     LIST (size) movi
 *         00dc (size)       <- sector aligned
 * RIFF (7F??????) AVIX      <- not more than 2GB in size
 *     JUNK (size)           <- fill to align to sector - 12
 *     LIST (size) movi
 *         00dc (size)       <- sector aligned
 *
 *-===================================================================*/

/* API to check if sufficient data is available in buffer */
ft_int32_t FT_AVIPCheckSz(FT_NBuffer_t *pnb,ft_int32_t OutLength)
{
	ft_int32_t	ValidLen;

	ValidLen = pnb->pEnd - pnb->pCurr;

	/* check if there is sufficient data in ring buffer, if not then read data into ring buffer */
	if(ValidLen < OutLength)
	{
		return 1;
	}
	return 0;
}


/* Layer for parsing - as of now using ringbuffer as scratch buffer implementation, later to be modified to Nbuffer for parallel */
//return status in case of EOF or fatal error etc
/* it is assumed that ringbuffer is filled when ever the valid size is 0, so write pointer is always pointing to the starting */
ft_int32_t FT_AVIPRead(FT_AVIParser_t *pAviP,ft_uint8_t *pOutBuffer,ft_int32_t OutLength, ft_int32_t *pReadLength)
{
	FT_NBuffer_t *pbuff;
	ft_int32_t	ValidLen;
	FT_AVIPARSER_STATUS Status = FT_AVIPARSER_OK;


	pbuff = &pAviP->SNbuff;
	ValidLen = pbuff->pEnd - pbuff->pCurr;

	/* check if there is sufficient data in ring buffer, if not then read data into ring buffer */
	if(ValidLen <= OutLength)
	{

		/* first read the partial chunk followed by next partial chunk */
		ft_uint32_t FirstPartial, SecondPartial;
		ft_int32_t fResult;

		FirstPartial = ValidLen;
		SecondPartial = OutLength - FirstPartial;

		//printf("Reload of next buffer %d %d %d %d %d %d\n",FirstPartial,SecondPartial,pbuff->pCurr,pbuff->pEnd,pbuff->Length,pbuff->pStart);
		if(FirstPartial > 0)
			memcpy(pOutBuffer,pbuff->pCurr,FirstPartial);

		pOutBuffer += FirstPartial;//increment wrt first partial
		*pReadLength = FirstPartial;

		/* fill the buffer */
#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
		FirstPartial = fread(pbuff->pStart, sizeof(ft_uint8_t), pbuff->Length, pAviP->File);
#elif defined(FT900_PLATFORM)
		fResult = f_read(&pAviP->File, pbuff->pStart, pbuff->Length, &FirstPartial);
#endif
		pbuff->pCurr = pbuff->pStart;
		pbuff->pEnd = pbuff->pStart + FirstPartial;

		if(FirstPartial < pbuff->Length)
		{
#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
			printf("File reading error: %d  reading pointer:%d \n", ferror(pAviP->File), ftell(pAviP->File));
			if (feof(pAviP->File))
				printf("EOF indicator detected.\n");
#endif
			Status = FT_AVIPARSER_EOF;
		}
#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
		else if (FirstPartial == 0){
			Status = FT_AVIPARSER_FILEERR;
		}
#elif defined(FT900_PLATFORM)
		else if(FR_OK != fResult)
		{
			Status = FT_AVIPARSER_FILEERR;
		}
#endif

		/* Fill the second partial data */
		if(SecondPartial > 0)
			memcpy(pOutBuffer,pbuff->pCurr,SecondPartial);

		pbuff->pCurr += SecondPartial;
		*pReadLength += SecondPartial;
		//printf("Reload done %d %d %d %d %x %d\n",FirstPartial,SecondPartial,pbuff->pCurr,pbuff->pEnd,*pbuff->pStart,f_tell(&pAviP->File));
	}
	else
	{
		memcpy(pOutBuffer,pbuff->pCurr,OutLength);
		pbuff->pCurr += OutLength;
		*pReadLength = OutLength;
	}

	return Status;
}

ft_uint8_t FT_AVIPRead8(FT_AVIParser_t *pAviP)
{
	FT_NBuffer_t *pbuff;
	pbuff = &pAviP->SNbuff;

	if((pbuff->pEnd - pbuff->pCurr) <= 1)
	{
		ft_uint8_t ReadByte;
		ft_int32_t Bytesread;

		FT_AVIPRead(pAviP,&ReadByte,1, &Bytesread);
		return (ReadByte);
	}
	else
	{

		return (*pbuff->pCurr++);
	}
}

ft_uint16_t FT_AVIPReadL16(FT_AVIParser_t *pAviP)
{
	FT_NBuffer_t *pbuff;
	ft_uint16_t Read16;

	pbuff = &pAviP->SNbuff;

	if((pbuff->pEnd - pbuff->pCurr) <= 2)
	{
		ft_int32_t Bytesread;

		FT_AVIPRead(pAviP,&Read16,2, &Bytesread);
	}
	else
	{
		Read16 = *pbuff->pCurr++;
		Read16 |= ((*pbuff->pCurr++) << 8);
	}
	return (Read16);
}

ft_uint32_t FT_AVIPReadL32(FT_AVIParser_t *pAviP)
{
	FT_NBuffer_t *pbuff;
	ft_uint32_t Read32;

	pbuff = &pAviP->SNbuff;

	if((pbuff->pEnd - pbuff->pCurr) <= 4)
	{
		ft_int32_t Bytesread;

		FT_AVIPRead(pAviP,&Read32,4, &Bytesread);
	}
	else
	{
		/* check if union approach gives more optimized code */
		Read32 = *pbuff->pCurr++;
		Read32 |= ((*pbuff->pCurr++) << 8);
		Read32 |= ((*pbuff->pCurr++) << 16);
		Read32 |= ((*pbuff->pCurr++) << 24);

	}

	//printf("READ32L %x \n",Read32);
	return (Read32);
}

ft_uint16_t FT_AVIPReadB16(FT_AVIParser_t *pAviP)
{
	FT_NBuffer_t *pbuff;
	ft_uint16_t Read16;

	pbuff = &pAviP->SNbuff;

	if((pbuff->pEnd - pbuff->pCurr) <= 2)
	{
		ft_uint8_t ReadArr[2];
		ft_int32_t Bytesread;

		FT_AVIPRead(pAviP,ReadArr,2, &Bytesread);
		Read16 = ((ReadArr[0]<<8) | (ReadArr[1]));
	}
	else
	{
		Read16 = ((*pbuff->pCurr++) << 8);
		Read16 |= *pbuff->pCurr++;
	}
	return (Read16);
}

ft_uint32_t FT_AVIPReadB32(FT_AVIParser_t *pAviP)
{
	FT_NBuffer_t *pbuff;
	ft_uint32_t Read32;

	pbuff = &pAviP->SNbuff;

	if((pbuff->pEnd - pbuff->pCurr) <= 4)
	{
		ft_uint8_t ReadArr[4];
		ft_int32_t Bytesread;

		FT_AVIPRead(pAviP,ReadArr,4, &Bytesread);
		Read32 = ((ReadArr[0]<<24) | (ReadArr[1]<<16) | (ReadArr[2]<<8) | ReadArr[3]);
	}
	else
	{
		Read32 = ((*pbuff->pCurr++) << 24);
		Read32 |= ((*pbuff->pCurr++) << 16);
		Read32 |= ((*pbuff->pCurr++) << 8);
		Read32 |= *pbuff->pCurr++;
	}
	return (Read32);
}
/* Fetch of fresh data from file - also taking care of sector alignment */
ft_int32_t FT_AVIPFetch(FT_AVIParser_t *pAviP,ft_uint32_t FileOffset)
{
	FT_AVIPARSER_STATUS Status = FT_AVIPARSER_OK;
	ft_uint32_t	Flushsector,freadsize;
	FT_NBuffer_t *pbuff;
	ft_int32_t fResult;
	
	pbuff = &pAviP->SNbuff;
	/* set current and end pointer respectively */
	pbuff->pCurr = pbuff->pStart;
	pbuff->pEnd = pbuff->pStart;

	Flushsector = (FileOffset & (~(FT_AVIP_SECTOR_SIZE - 1)));//assuming sector size if 2 power n

	//printf("AVIPFetch %d %d %d\n",ValidLen,FileOffset,Flushsector);
#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
	fResult = fseek(pAviP->File, Flushsector, SEEK_SET);
	if(fResult != 0){
		Status = FT_AVIPARSER_FSEEKERR;
	}
	
	freadsize = fread(pbuff->pStart, sizeof(ft_uint8_t),pbuff->Length,pAviP->File);
	if(freadsize != pbuff->Length){
		Status = FT_AVIPARSER_FREADERR;
	}
#elif defined(FT900_PLATFORM)
	fResult = f_lseek(&pAviP->File,Flushsector);
	if(FR_OK != fResult)
	{
		Status = FT_AVIPARSER_FSEEKERR;
	}
	fResult = f_read(&pAviP->File, pbuff->pStart, pbuff->Length, &freadsize);
	if(FR_OK != fResult)
	{
		Status = FT_AVIPARSER_FREADERR;
	}
#endif
	pbuff->pCurr += (FileOffset - Flushsector);//change the offset to current position
	pbuff->pEnd = pbuff->pStart + freadsize;
	//printf("AVIPFetch done %d %d %d %d \n",Flushsector,pbuff->pCurr,f_tell(&pAviP->File),*pbuff->pCurr);
	
	return Status;
}
/* API to flush data from local buffer and increment wrt file */
ft_int32_t FT_AVIPFlush(FT_AVIParser_t *pAviP,ft_int32_t FlushLength)
{
	FT_NBuffer_t *pbuff;
	ft_int32_t	ValidLen,Flushsector;
	FT_AVIPARSER_STATUS Status = FT_AVIPARSER_OK;
	ft_int32_t fResult;

	pbuff = &pAviP->SNbuff;
	ValidLen = pbuff->pEnd - pbuff->pCurr;

	//printf("AVIPFlush %d %d %d \n",ValidLen,pbuff->pCurr,pbuff->pEnd);
	/* Check if flush size is within the present buffer, if yes then increment the current pointer, if not then flush whole and seek to sector aligned location and read the whole buffer and point the current buffer to valid location */
	if(FlushLength <= ValidLen)
	{

		pbuff->pCurr += FlushLength;
	}
	else
	{
		ft_uint32_t freadsize;
		/* Check how much is to be flushed */
		FlushLength -= ValidLen;
		pbuff->pCurr = pbuff->pStart;
		pbuff->pEnd = pbuff->pStart;

		Flushsector = (FlushLength & (~(FT_AVIP_SECTOR_SIZE - 1)));//assuming sector size if 2 power n

		//printf("AVIPFlush %d %d %d %d\n",ValidLen,FlushLength,Flushsector,Flushsector + f_tell(&pAviP->File));
#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
		fResult = fseek(pAviP->File, Flushsector + ftell(pAviP->File), SEEK_SET);
		if(fResult != 0){
			Status = FT_AVIPARSER_FSEEKERR;
		}

		freadsize = fread(pbuff->pStart, sizeof(ft_uint8_t),pbuff->Length,pAviP->File);
		if(freadsize != pbuff->Length){
			Status = FT_AVIPARSER_FREADERR;
		}

#elif defined(FT900_PLATFORM)
		fResult = f_lseek(&pAviP->File,Flushsector + f_tell(&pAviP->File));
		if(FR_OK != fResult)
		{
			Status = FT_AVIPARSER_FSEEKERR;
		}
		fResult = f_read(&pAviP->File, pbuff->pStart, pbuff->Length, &freadsize);
		if(FR_OK != fResult)
		{
			Status = FT_AVIPARSER_FREADERR;
		}
#endif
		pbuff->pCurr += (FlushLength - Flushsector);//change the offset to current position
		pbuff->pEnd = pbuff->pStart + freadsize;
		//printf("AVIPFlush done %d %d %d %d %d\n",Flushsector,pbuff->pCurr,f_tell(&pAviP->File),*pbuff->pCurr,freadsize);
	}

	return Status;
}
/* Initialize AVI parser */
ft_int32_t	FT_AVIParser_Init(FT_AVIParser_t *pCtxt)
{
	/* Initialize all the parameters of parser context */

	FT_AVIPARSER_STATUS Status = FT_AVIPARSER_OK;

	pCtxt->NumRiff = 0;
	pCtxt->NumMedia = 0;
	pCtxt->CurrMediaType = 0;
	pCtxt->FileSz = 0;
	pCtxt->Flags = 0;
	pCtxt->CurrRiff;

	/* Buffer initialization */
	pCtxt->SNbuff.Length = 0;
	pCtxt->SNbuff.pCurr = NULL;
	pCtxt->SNbuff.pEnd = NULL;
	pCtxt->SNbuff.pNext = NULL;
	pCtxt->SNbuff.pStart = NULL;

	pCtxt->CurrStat.IndxOff = 0;
	pCtxt->CurrStat.MoviOff = 0;

	memset(pCtxt->Riff,sizeof(FT_AVIRiff_t)*FT_AVIP_MAXRIFFHDLS,0);
	/* File pointer - specific to a file system */
	//pCtxt->File = NULL;
	pCtxt->SVid.CurrFrame = 0;
	pCtxt->SVid.TotFrames = 0;
	pCtxt->SVid.Flags = 0;
	
	pCtxt->SAud.CurrFrame = 0;
	pCtxt->SAud.TotFrames = 0;
	pCtxt->SAud.packet_size = 0;
	
	return Status;
}

/* Set Nbuffer parameters */
ft_int32_t	FT_AVIParser_SetBuff(FT_AVIParser_t *pCtxt,ft_uint8_t *pbuffer,ft_int32_t BuffLen)
{
	FT_NBuffer_t *pnb;

	if(BuffLen <= 0)
	{
		return FT_AVIPARSER_INVALIDPARAMS;
	}

	pnb = &pCtxt->SNbuff;

	pnb->Length = BuffLen;
	pnb->pStart = pbuffer;
	pnb->pCurr = pbuffer;
	pnb->pEnd = pbuffer;

	pnb->pNext = NULL;//hack - as of now N buffers are not used, only 1 buffer is used

	return FT_AVIPARSER_OK;
}

/* Set file pointer - open the  */
ft_int32_t	FT_AVIParser_SetOpenFile(FT_AVIParser_t *pCtxt,ft_char8_t *pFilename)
{
	ft_int32_t fResult;

#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
	pCtxt->File = fopen(pFilename, "rb");

	if(pCtxt->File != NULL){
		printf("\n open video successed!\n");

	}
	else{
		printf("Unable to open: %s\n",pFilename);
		return FT_AVIPARSER_FILENOTFOUND;
	}
#elif defined(FT900_PLATFORM)
	fResult = f_open(&pCtxt->File, pFilename, FA_READ);

	if(FR_OK != fResult)
	{
		printf("\n open video successed!\n");
		return FT_AVIPARSER_FILENOTFOUND;
	}
#endif

	pCtxt->NumRiff = 0;
#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)

	fseek(pCtxt->File, 0L, SEEK_END);
	pCtxt->FileSz = ftell(pCtxt->File);
	fseek(pCtxt->File, 0L, SEEK_SET);
#elif defined(FT900_PLATFORM)
	pCtxt->FileSz = f_size(&pCtxt->File);
#endif

	pCtxt->Flags = 0;
	
	/* Update the file related parameters in the AVIparser context */
	return FT_AVIPARSER_OK;
}

/* Close file, reset buffer pointers etc */
ft_int32_t	FT_AVIParser_Reset(FT_AVIParser_t *pCtxt,ft_char8_t *pFilename)
{
	ft_int32_t fResult;

	FT_AVIParser_Init(pCtxt);

#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)	
	fResult = fclose(pCtxt->File);
	if(fResult != 0){
		return FT_AVIPARSER_FILENOTFOUND;
	}
#elif defined(FT900_PLATFORM)
	fResult = f_close(&pCtxt->File);
	if(FR_OK != fResult)
	{
		return FT_AVIPARSER_FILENOTFOUND;
	}
#endif
	return FT_AVIPARSER_OK;
}
//set audio only, video only, fast seek, drop video frames, fakesync
ft_int32_t	FT_AVIParser_SetProp(FT_AVIParser_t *pCtxt);
//properties of overall stream, audio, video, text etc,
ft_int32_t	FT_AVIParser_GetProp(FT_AVIParser_t *pCtxt);
//parse the whole avi file - to be done at the starting of the playback
ft_int32_t	FT_AVIParser_ParseAvi(FT_AVIParser_t *pCtxt)
{
	FT_AVIPARSER_STATUS Status = FT_AVIPARSER_OK;
	ft_uint32_t Next4cc, ChunkSz, AviS,List4cc;
	FT_NBuffer_t *pnb;
	
	/* For safer side make sure the current file pointer is at the begining of the file */
#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
	fseek(pCtxt->File, 0, SEEK_SET);
#elif defined(FT900_PLATFORM)
	f_lseek(&pCtxt->File,0);
#endif
	Next4cc = FT_AVIPReadL32(pCtxt);
	ChunkSz = FT_AVIPReadL32(pCtxt);
	AviS = FT_AVIPReadL32(pCtxt);
	pCtxt->FileSz -= 16;//read 3 words

	//printf("New file %x %x %x\n",Next4cc,ChunkSz,AviS);
	pnb = &pCtxt->SNbuff;

	/* First check for RIFF chunk, followed by parsing of all the media properties */
	if((FT_AVI_4CC_RIFF != Next4cc) || (FT_AVI_4CC_AVI != AviS))
	{
		printf("Error, not an AVI file %x %x %x\n",Next4cc,ChunkSz,AviS);
		return FT_AVIPARSER_NOTAVIFILE;
	}

#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
	pCtxt->Riff[pCtxt->NumRiff].RiffOff = ftell(pCtxt->File) - (pnb->pEnd - pnb->pCurr);   ///QUESTION: not sure what is the purpose? should the offset be the location of the movi list?
#elif defined(FT900_PLATFORM)
	pCtxt->Riff[pCtxt->NumRiff].RiffOff = f_tell(&pCtxt->File) - (pnb->pEnd - pnb->pCurr);
#endif
	pCtxt->Riff[pCtxt->NumRiff].RiffSize = ChunkSz - 4;//already avi is consumed
	/* update the riff handles in parser */
	pCtxt->NumRiff++;
	/* Parse all the headers, log the values, update the avi parser context */

	//while(pCtxt->FileSz > 0)//will not work in case of streaming
	while(1)//will not work in case of streaming
	{
#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
		if (feof(pCtxt->File))  
		{
#elif defined(FT900_PLATFORM)
		if(0 != f_eof(&pCtxt->File))
		{
#endif
			printf("EOF reached \n");
			return Status;
			//goto parser_error;  ///original
		}
		Next4cc = FT_AVIPReadL32(pCtxt);
		ChunkSz = FT_AVIPReadL32(pCtxt);

		//printf("New chunk/list %x %x\n",Next4cc,ChunkSz);
		switch(Next4cc)
		{
			case FT_AVI_4CC_RIFF:
			{
				/* check for AVIX */
				//printf("RIFF AVIX\n");
				AviS = FT_AVIPReadL32(pCtxt);
				if(AviS != FT_AVI_4CC_AVIX)
				{
					goto parser_error;
				}
#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)	
				pCtxt->Riff[pCtxt->NumRiff].RiffOff = ftell(pCtxt->File) - (pnb->pEnd - pnb->pCurr);
#elif defined(FT900_PLATFORM)
				pCtxt->Riff[pCtxt->NumRiff].RiffOff = f_tell(&pCtxt->File) - (pnb->pEnd - pnb->pCurr);
#endif
				pCtxt->Riff[pCtxt->NumRiff].RiffSize = ChunkSz - 4;//avix is already read from buffer
				//printf("RIFF AVIX %x %x\n",pCtxt->Riff[pCtxt->NumRiff].RiffOff,pCtxt->Riff[pCtxt->NumRiff].RiffSize);

				/* increment the riff index */
				pCtxt->NumRiff++;
				break;
			}
			case FT_AVI_4CC_LIST:
			{
				ft_uint32_t list4cc = FT_AVIPReadL32(pCtxt);
				//printf("New list ele %x\n",list4cc);
				switch(list4cc)
				{
					case FT_AVI_4CC_hdrl:
					{
						/* next parse avih */
						//printf("hdrl list\n");
						break;
					}
					case FT_AVI_4CC_strl:
					{
						//printf("strl list\n");
						pCtxt->CurrMediaType = 0;
						/* check if audio or video */
						break;
					}
					case FT_AVI_4CC_odml:
					{
						//printf("odml list\n");
						/* AVI 2.0 */
						/* skip all the info list */
						FT_AVIPFlush(pCtxt,ChunkSz - 4);
						break;
					}
					case FT_AVI_4CC_INFO:
					{
						//printf("info list\n");
						/* skip all the info list */
						FT_AVIPFlush(pCtxt,ChunkSz - 4);
						break;
					}
					case FT_AVI_4CC_movi:
					{
						FT_AVIRiff_t *pmovi;

						//printf("movi list %x %x %x\n",f_tell(&pCtxt->File),pnb->pEnd,pnb->pCurr);

						pmovi = &pCtxt->Riff[pCtxt->NumRiff - 1];//need to ensure that RIFF is parsed before movi
#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
						pmovi->MoviOff = ftell(pCtxt->File) - (pnb->pEnd - pnb->pCurr);
#elif defined(FT900_PLATFORM)
						pmovi->MoviOff = f_tell(&pCtxt->File) - (pnb->pEnd - pnb->pCurr);//absolute value of movi in 4 gb file
#endif
						pmovi->MoviSize = ChunkSz - 4;
						//pmovi->MoviOff = 0;//this is relative to movi
						//printf("movi list %x %x %x \n",pCtxt->NumRiff,pmovi->MoviOff,pmovi->MoviSize);


						FT_AVIPFlush(pCtxt,ChunkSz - 4);
						break;
					}
					default:
					{
						printf("Error default for list %x\n",list4cc);
						FT_AVIPFlush(pCtxt,ChunkSz - 8);
						break;
					}
				}
				break;
			}
			case FT_AVI_4CC_avih:
			{
				//printf("avih \n");
				FT_AVIPReadL32(pCtxt);
				FT_AVIPReadL32(pCtxt);
				FT_AVIPReadL32(pCtxt);
				pCtxt->SVid.Flags = FT_AVIPReadL32(pCtxt);
				FT_AVIPReadL32(pCtxt);
				FT_AVIPReadL32(pCtxt);
				FT_AVIPReadL32(pCtxt);
				FT_AVIPReadL32(pCtxt);
				pCtxt->SVid.Width = FT_AVIPReadL32(pCtxt);
				pCtxt->SVid.Height = FT_AVIPReadL32(pCtxt);
				FT_AVIPReadL32(pCtxt);
				FT_AVIPReadL32(pCtxt);
				FT_AVIPReadL32(pCtxt);
				FT_AVIPReadL32(pCtxt);
				printf("avih width height %d %d\n",pCtxt->SVid.Width,pCtxt->SVid.Height);
				break;
			}
			case FT_AVI_4CC_strh:
			{
				ft_uint32_t fcctype,fcchandler;

				//printf("strh \n");
				fcctype = FT_AVIPReadL32(pCtxt);
				{
					fcchandler = FT_AVIPReadL32(pCtxt);
					FT_AVIPReadL32(pCtxt);
					FT_AVIPReadL32(pCtxt);
					FT_AVIPReadL32(pCtxt);
					if(fcctype == FT_AVI_4CC_vids)
					{
						//printf("strh video\n");
						pCtxt->SVid.TimeScale = FT_AVIPReadL32(pCtxt);
						pCtxt->SVid.DataRate = FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL32(pCtxt);
						pCtxt->SVid.TotFrames = FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL32(pCtxt);
						pCtxt->SVid.SampleSize = FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL32(pCtxt);
						pCtxt->CurrMediaType = 1;
						printf("strh video %x %x %x %x \n",pCtxt->SVid.TimeScale,pCtxt->SVid.DataRate,pCtxt->SVid.TotFrames,pCtxt->SVid.SampleSize);
					}
					else if(fcctype == FT_AVI_4CC_auds)
					{
						//printf("strh audio\n");
						pCtxt->SAud.TimeScale = FT_AVIPReadL32(pCtxt);
						pCtxt->SAud.DataRate = FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL32(pCtxt);
						pCtxt->SAud.TotFrames = FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL32(pCtxt);
						pCtxt->SAud.SampleSize = FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL32(pCtxt);
						pCtxt->CurrMediaType = 2;
						//printf("strh audio %x %x %x %x \n",pCtxt->SAud.TimeScale,pCtxt->SAud.DataRate,pCtxt->SAud.TotFrames,pCtxt->SAud.SampleSize);
					}
					else
					{
						//printf("strh txt or midi \n");
						/* could be text or midi */
						FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL32(pCtxt);
					}
				}
				break;
			}
			case FT_AVI_4CC_strf:
			{
				//printf("strf\n");
				if(1 == pCtxt->CurrMediaType)
				{
					/* parse bitmap header */
					ft_uint32_t strsize = FT_AVIPReadL32(pCtxt);
					FT_AVIPReadL32(pCtxt);
					FT_AVIPReadL32(pCtxt);
					FT_AVIPReadL32(pCtxt);
					FT_AVIPReadL32(pCtxt);
					FT_AVIPReadL32(pCtxt);
					FT_AVIPReadL32(pCtxt);
					FT_AVIPReadL32(pCtxt);
					FT_AVIPReadL32(pCtxt);
					FT_AVIPReadL32(pCtxt);
					if((strsize - 4*10) != 0)
					{
						//printf("video hdr has extra data %d %d\n",strsize);
						FT_AVIPFlush(pCtxt,strsize - 4*10);
					}
#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
					pCtxt->tempstore = ftell(pCtxt->File) - (pCtxt->SNbuff.pEnd - pCtxt->SNbuff.pCurr);
#elif defined(FT900_PLATFORM)
					pCtxt->tempstore = f_tell(&pCtxt->File) - (pCtxt->SNbuff.pEnd - pCtxt->SNbuff.pCurr);
#endif
				}
				else if(2 == pCtxt->CurrMediaType)
				{
					ft_uint32_t cbSize,strsize = FT_AVIPReadL16(pCtxt);//wave format

					//printf("strf audio waveformat %x\n",strsize);

					if((strsize >= FT_WAVE_FORMAT_PCM) && (strsize <= FT_WAVE_FORMAT_MULAW))
					{
						/* Parse as wave WAVEFORMATEX */
						pCtxt->SAud.NumChannels = FT_AVIPReadL16(pCtxt);
						pCtxt->SAud.SamplingFreq = FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL16(pCtxt);
						pCtxt->SAud.BitsPerSample = FT_AVIPReadL16(pCtxt);

						cbSize = FT_AVIPReadL16(pCtxt);
						//printf("audio WAVEFORMATEX has extra data %d %d %d %d\n",pCtxt->SAud.NumChannels,pCtxt->SAud.SamplingFreq,pCtxt->SAud.BitsPerSample,cbSize);
						if(cbSize > 0)
						{
							FT_AVIPFlush(pCtxt,cbSize);
						}
					}
					if(FT_WAVE_FORMAT_MPEGLAYER3 == strsize)
					{
						//printf("strf audio mp3 %x\n",strsize);
						/* parse mp3 header */
						pCtxt->SAud.NumChannels = FT_AVIPReadL16(pCtxt);
						pCtxt->SAud.SamplingFreq = FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL16(pCtxt);
						pCtxt->SAud.BitsPerSample = FT_AVIPReadL16(pCtxt);

						cbSize = FT_AVIPReadL16(pCtxt);

						FT_AVIPReadL16(pCtxt);
						FT_AVIPReadL32(pCtxt);
						FT_AVIPReadL16(pCtxt);
						FT_AVIPReadL16(pCtxt);
						FT_AVIPReadL16(pCtxt);
						//printf("audio mp3 %d %d %d\n",pCtxt->SAud.NumChannels,pCtxt->SAud.SamplingFreq,pCtxt->SAud.BitsPerSample);
						//cbSize = FT_AVIPReadL16(pCtxt);
					}
				}

				break;
			}
			case FT_AVI_4CC_idx1:
			{
				/* Update index entry points for seek purpose */
				FT_AVIRiff_t *pidx1;

				//printf("idx1\n");
				pidx1 = &pCtxt->Riff[pCtxt->NumRiff - 1];//need to ensure that RIFF is parsed before movi
#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
				pidx1->indxOff = ftell(pCtxt->File) - (pnb->pEnd - pnb->pCurr);
#elif defined(FT900_PLATFORM)
				pidx1->indxOff = f_tell(&pCtxt->File) - (pnb->pEnd - pnb->pCurr);//absolute value of movi in 4 gb file
#endif		
				pidx1->indxSize = ChunkSz;

				FT_AVIPFlush(pCtxt,ChunkSz);
				break;
			}
			case FT_AVI_4CC_JUNK:
			case FT_AVI_4CC_vprp:
			case FT_AVI_4CC_indx:
			{
				printf("extra indx/junk/vprp \n");

				FT_AVIPFlush(pCtxt,ChunkSz);
				break;
			}
			default:
			{
				printf("Error default for 4cc %x\n",Next4cc);
				FT_AVIPFlush(pCtxt,ChunkSz);
				break;
			}
		}
	}

	if(pCtxt->NumMedia == 0)
	{
		Status = FT_AVIPARSER_INVALIDFILE;
	}


	return Status;

parser_error:
	return FT_AVIPARSER_UNKNOWNERR;
}

//parser only one RIFF and fill the context respectively
ft_int32_t	FT_AVIParser_ParseAviRiff(FT_AVIParser_t *pCtxt);

/* Set the RIFF to be played */
ft_int32_t	FT_AVIParser_SetRiff(FT_AVIParser_t *pCtxt,ft_uint8_t RiffNo)
{
	
	if(RiffNo > pCtxt->NumRiff)
	{
		return FT_AVIPARSER_INVALIDPARAMS;
	}
	
	/* Set all the parameters for current RIFF - need to add support for multiple riffs during runtime */
	pCtxt->CurrRiff = RiffNo;
	//pCtxt->CurrStat.MoviOff = pCtxt->Riff[pCtxt->CurrRiff].MoviOff;
	//pCtxt->CurrStat.IndxOff = pCtxt->Riff[pCtxt->CurrRiff].indxOff;
	pCtxt->CurrStat.MoviOff = 0;//current riff chunks movi offset
	pCtxt->CurrStat.IndxOff = 0;//current riff chunks idx1 offset
	
	/* Initialize the Nbuffer with first video data */
	FT_AVIPFetch(pCtxt,pCtxt->Riff[pCtxt->CurrRiff].MoviOff);
	
	printf("Aviriff set %d %x %x \n",pCtxt->CurrRiff,pCtxt->Riff[pCtxt->CurrRiff].MoviOff,pCtxt->Riff[pCtxt->CurrRiff].MoviSize);
	/* make sure all the audio/video metadata and data are flushed properly */
	
	return FT_AVIPARSER_OK;
}

/* Fill AudLen/VidLen length completely - always min(AudLen, VidLen) is done */
ft_int32_t	FT_AVIParser_FillChunks(FT_AVIParser_t *pCtxt,FT_VidDataPath_t *pvdp,FT_AudDataPath_t *padp,FT_AviFillCkhs_t *pFChks)
{
	FT_AVIPARSER_STATUS Status = FT_AVIPARSER_OK;

	/* as of now only video buffer size is loaded */
	return Status;
}

/* API to download data into ft81x input buffer directly - fifo ring buffer mechanism */
/* No avi parsing done, blindly tries to copy data from cache buffer/file into FT81x memory */
ft_int32_t FT_AVIFillVdata(FT_AVIParser_t *pCtxt,FT_VidDataPath_t *pvdp,ft_uint32_t VDatatocopy)
{
	Ft_Fifo_t *pVFifo;
	Ft_Gpu_Hal_Context_t *pHHal;
	ft_uint32_t CurrszAVIPRB;
	ft_int32_t numbyteswritten = 0;
	
	/* FT81x specific pointers */
	pHHal = pvdp->phalctxt;
	pVFifo = pvdp->pfifo;
	
	if(VDatatocopy <= 0)
	{
		/* sanity check */
		//printf("Error value is %x\n",VDatatocopy);
		return FT_AVIPARSER_INVALIDPARAMS;
	}
	
	/* Direct modification for the ring buffer */
	while(VDatatocopy > 0)
	{
		ft_uint32_t freadsize,fResult;
		
		/* Direct modification */
		CurrszAVIPRB = pCtxt->SNbuff.pEnd - pCtxt->SNbuff.pCurr;
		if(CurrszAVIPRB < VDatatocopy)
		{
			/* Blocking call to download data into ft81x ring buffer - copy only and not decode */

			numbyteswritten = Ft_Fifo_Write(pHHal,pVFifo,pCtxt->SNbuff.pCurr,CurrszAVIPRB);//hack video fake sync
			//printf("VFill parbuff %x %x %x\n",Ft_Gpu_Hal_Rd32(pHHal,REG_MEDIAFIFO_READ),Ft_Gpu_Hal_Rd32(pHHal,REG_MEDIAFIFO_WRITE),CurrszAVIPRB);
			VDatatocopy -= numbyteswritten;
			
			if(numbyteswritten == CurrszAVIPRB)
			{
#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
				freadsize = fread(pCtxt->SNbuff.pStart, sizeof(ft_uint8_t), pCtxt->SNbuff.Length, pCtxt->File);
				if(freadsize != pCtxt->SNbuff.Length){
					return FT_AVIPARSER_FREADERR;
				}
#elif defined(FT900_PLATFORM)
				//timer_profilecountstart(&timercount,&timervalue);
				fResult = f_read(&pCtxt->File, pCtxt->SNbuff.pStart, pCtxt->SNbuff.Length, &freadsize);
				//timer_profilecountend(&timercount,&timervalue);
				if(FR_OK != fResult)
				{
					return FT_AVIPARSER_FREADERR;
				}
#endif
				pCtxt->SNbuff.pCurr = pCtxt->SNbuff.pStart;
				pCtxt->SNbuff.pEnd = pCtxt->SNbuff.pCurr + freadsize;
			}
			else
			{
				pCtxt->SNbuff.pCurr += numbyteswritten;

			}
		}
		else
		{
			/* Non blocking from wait point of view but blocking from copy point of view */

			numbyteswritten = Ft_Fifo_Write(pHHal,pVFifo,pCtxt->SNbuff.pCurr,VDatatocopy);//hack for video fake sync
			/* dump the bytes for sanity check */
			{
				ft_uint32_t tempraw[8];
				Ft_Gpu_Hal_RdMem(pHHal,pVFifo->fifo_buff + pVFifo->fifo_rp,tempraw,4*4);
				Ft_Gpu_Hal_RdMem(pHHal,pVFifo->fifo_buff + pVFifo->fifo_wp - 4*4,&tempraw[4],4*4);
				//printf("rawdata %x %x %x %x %x %x %x %x\n",tempraw[0],tempraw[1],tempraw[2],tempraw[3],tempraw[4],tempraw[5],tempraw[6],tempraw[7]);
			}

			/* handling 4 bytes alignment for imagedecode */

			if(pVFifo->fifo_wp & 0x03)
			{
				ft_uint8_t paddlen = pVFifo->fifo_wp & 0x03;
				ft_uint32_t padddata = 0;

				Ft_Fifo_Write(pHHal,pVFifo,&padddata,paddlen);
			}
			//printf("VFill %x %x %x\n",Ft_Gpu_Hal_Rd32(pHHal,REG_MEDIAFIFO_READ),Ft_Gpu_Hal_Rd32(pHHal,REG_MEDIAFIFO_WRITE),VDatatocopy);
			pCtxt->SNbuff.pCurr	+= numbyteswritten;
			VDatatocopy -= numbyteswritten;
			
		}
	}	
	return FT_AVIPARSER_OK;
}
/* API to download data input audio input buffer - ring buffer mechanism */
/* No avi parsing done, blindly tries to copy data from cache buffer/file into FT81x memory */
ft_int32_t FT_AVIFillAdata(FT_AVIParser_t *pCtxt,FT_AudDataPath_t *padp,ft_uint32_t ADatatocopy)
{
	ft_uint32_t CurrszAVIPRB;
	
	if(ADatatocopy <= 0)
	{
		/* sanity check */
		//printf("Error value is %x\n",ADatatocopy);
		return FT_AVIPARSER_INVALIDPARAMS;
	}
	
	/* Direct modification for the ring buffer */
	while(ADatatocopy > 0)
	{
		ft_uint32_t freadsize,fResult;
		
		CurrszAVIPRB = pCtxt->SNbuff.pEnd - pCtxt->SNbuff.pCurr;
		if(CurrszAVIPRB < ADatatocopy)
		{
			/* Blocking call to copy data into audio input buffer */
			//FT_RingBuffer_Write(padp->pairb,pCtxt->SNbuff.pCurr,CurrszAVIPRB,&freadsize);//hack for audio fake sync
			freadsize = CurrszAVIPRB;//hack for audio fake sync

			if(freadsize != CurrszAVIPRB)
			{
				printf("Error whole data is not copied into rb %d %d\n",freadsize,CurrszAVIPRB);
				return FT_AVIPARSER_COPYERROR;
			}
			
			ADatatocopy -= CurrszAVIPRB;					
#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
			freadsize = fread(pCtxt->SNbuff.pStart, sizeof(ft_uint8_t), pCtxt->SNbuff.Length, pCtxt->File);
			if(freadsize != pCtxt->SNbuff.Length){
				return FT_AVIPARSER_FREADERR;
			}
#elif defined(FT900_PLATFORM)
			fResult = f_read(&pCtxt->File, pCtxt->SNbuff.pStart, pCtxt->SNbuff.Length, &freadsize);
			if(FR_OK != fResult)
			{
				return FT_AVIPARSER_FREADERR;
			}
#endif
			pCtxt->SNbuff.pCurr = pCtxt->SNbuff.pStart;
			pCtxt->SNbuff.pEnd = pCtxt->SNbuff.pCurr + freadsize;
		}
		else
		{
			/* None blocking from wait point of view but blocking from copy point of view */
			/* Blocking call to copy data into audio input buffer */
			//FT_RingBuffer_Write(padp->pairb,pCtxt->SNbuff.pCurr,ADatatocopy,&freadsize);//hack for audio fake sync
			freadsize = ADatatocopy;//hack for audio fake sync

			if(freadsize != ADatatocopy)
			{
				printf("Error whole data is not copied into rb %d %d\n",freadsize,CurrszAVIPRB);
				return FT_AVIPARSER_COPYERROR;
			}
			pCtxt->SNbuff.pCurr	+= ADatatocopy;
			ADatatocopy = 0;
		}
	}
	
	return FT_AVIPARSER_OK;
}

/* Check for partial buffer case and fill data */
ft_int32_t FT_AVIPVidChkFillParBuff(FT_AVIParser_t *pCtxt,FT_VidDataPath_t *pvdp)
{
	FT_RingQueue_t *prqv;
	ft_int32_t Status = FT_AVIPARSER_OK;
	Ft_Gpu_Hal_Context_t *pHHal;
	Ft_Fifo_t *pVFifo;

	prqv = pvdp->pvirq;
	pHHal = pvdp->phalctxt;
	pVFifo = pvdp->pfifo;
	/* Check for skip frame flag from player */
	/* Copy data from parser to video ring buffer in case of partial buffer */
	if(prqv->ValidEle > 0 )
	{
		ft_int32_t VInputRingb,VDatatocopy = 0;
		FT_MetaV_t SMetaV;

		FT_RQueue_PeekELatest(prqv,&SMetaV);
		//printf("Videochk partial %x %x %x %x %x %x %x %x %x\n",SMetaV.Length,SMetaV.PartialSz,SMetaV.Flags,prqv->WriteIdx,prqv->ValidEle,Ft_Gpu_Hal_Rd32(pHHal,REG_MEDIAFIFO_READ),Ft_Gpu_Hal_Rd32(pHHal,REG_MEDIAFIFO_WRITE),Ft_Gpu_Hal_Rd32(pHHal,REG_CMD_READ),Ft_Gpu_Hal_Rd32(pHHal,REG_CMD_WRITE));
		/* In case of partial frame, first download the partial frame */
		if(SMetaV.Flags & FT_AVIPMETAD_PARTIALBUFFER)
		{
			Status = FT_AVIPARSER_PARTIALBUFFERUSECASE;

			/* Get the available free size in FT81x */
			VInputRingb = Ft_Fifo_GetFreeSpace(pHHal,pVFifo);
			//VInputRingb = FT_AVIPARSER_TESTSTUB_VIDBUFFMAXSZ;//hack for video fake sync
			//printf("Videochk partial %x %x %x %x\n",SMetaV.Length,SMetaV.PartialSz,SMetaV.Flags,VInputRingb);
			/* Partial buffer case where input buffer has no space, so the chunk size present in meta should be used */
			VDatatocopy = SMetaV.Length - SMetaV.PartialSz;
			if(VInputRingb < VDatatocopy)
			{
				/* again partial scenarios, so copy partial and re update the metadata element */
				VDatatocopy = VInputRingb;
				SMetaV.PartialSz += VInputRingb;
			}
			else
			{
				SMetaV.Flags &= (~FT_AVIPMETAD_PARTIALBUFFER);
				SMetaV.PartialSz = 0;
			}
			if(VDatatocopy > 0)
			{
				Status = FT_AVIFillVdata(pCtxt, pvdp,VDatatocopy);
				if(Status != FT_AVIPARSER_OK)
				{
					return Status;
				}
			}

			FT_RQueue_WriteEInplace(prqv,&SMetaV);


			/* return partial buffer usecase or error from fillvdata */
			return FT_AVIPARSER_PARTIALBUFFERUSECASE;
		}
	}
	return Status;
}

/* Check for partial buffer case and fill data */
ft_int32_t FT_AVIPAudChkFillParBuff(FT_AVIParser_t *pCtxt,FT_AudDataPath_t *padp)
{
	FT_RingQueue_t *prqa;
	ft_int32_t Status = FT_AVIPARSER_OK;


	prqa = padp->pairq;

	/* Copy data from parser to audio input ring buffer in case of partial buffer */
	/* Check for skip frame flag settings */
	if(prqa->ValidEle > 0 )
	{
		ft_int32_t AInputRingb,ADatatocopy = 0;
		FT_MetaA_t SMetaA;

		FT_RQueue_PeekELatest(prqa,&SMetaA);
		//printf("Audiochk partial %x %x %x %x %x\n",SMetaA.Length,SMetaA.PartialSz,SMetaA.Flags,prqa->WriteIdx,prqa->ValidEle);
		/* In case of partial frame, first download the partial frame */
		if(SMetaA.Flags & FT_AVIPMETAD_PARTIALBUFFER)
		{
			Status = FT_AVIPARSER_PARTIALBUFFERUSECASE;

			/* Get the free size from pcm/mp3 codec */
			//printf("Audiochk partial %x %x %x %x %x\n",SMetaA.Length,SMetaA.PartialSz,SMetaA.Flags,prqa->WriteIdx,prqa->ValidEle);
			//AInputRingb = FT_RingBuffer_FreeSpace(prba);
			AInputRingb = FT_AVIPARSER_TESTSTUB_AUDBUFFMAXSZ;//hack for audio fake sync

			/* Partial buffer case where input buffer has no space, so the chunk size present in meta should be used */
			ADatatocopy = SMetaA.Length - SMetaA.PartialSz;
			if(AInputRingb < ADatatocopy)
			{
				/* again partial scenarios, so copy partial and re update the metadata element */
				ADatatocopy = AInputRingb;
				SMetaA.PartialSz += ADatatocopy;
			}
			else
			{
				SMetaA.Flags &= (~FT_AVIPMETAD_PARTIALBUFFER);
				SMetaA.PartialSz = 0;
			}
			if(ADatatocopy > 0)
			{
				Status = FT_AVIFillAdata(pCtxt, padp,ADatatocopy);
				if(Status != FT_AVIPARSER_OK)
				{
					return Status;
				}

			}

			FT_RQueue_WriteEInplace(prqa,&SMetaA);

			/* return after partial chunk */
			return FT_AVIPARSER_PARTIALBUFFERUSECASE;
		}
	}
	
	return Status;
}


ft_int32_t FT_AVIPVidFillChunk(FT_AVIParser_t *pCtxt,FT_VidDataPath_t *pvdp,ft_uint32_t VBuffSz)
{
	ft_uint32_t VidHdr;
	FT_MetaV_t SMetaV;
	ft_int32_t VDatatocopy;
	FT_RingQueue_t *prqv;
	FT_AviV_t *paviv;
	Ft_Fifo_t *pVFifo;
	Ft_Gpu_Hal_Context_t *pHHal;

	/* video specific local pointers */
	prqv = pvdp->pvirq;
	paviv = &pCtxt->SVid;

	/* FT81x specific pointers */
	pHHal = pvdp->phalctxt;
	pVFifo = pvdp->pfifo;

	FT_MetaV_Init(&SMetaV);

	/* Check for skip frames in case of video */
	if(pCtxt->Flags & (FT_AVIPARSER_DROPVIDEO | FT_AVIPARSER_DROPALLMEDIA | FT_AVIPARSER_FAKESYNC))
	{
		//printf("Videochk flush %x %x \n",chksz,pCtxt->Flags);
		FT_AVIPFlush(pCtxt,VBuffSz);//fake sync
		FT_RQueue_WriteE(prqv,&SMetaV);//incorrect in case of partial frames present in chunk
		paviv->CurrFrame++;//incorrect in case of partial frames present in chunk
		return FT_AVIPARSER_OK;//do not insert any metadata
	}

	VDatatocopy = 0;
	if(VBuffSz > 0)
	{
		FT_NBuffer_t *pnbuff;
		ft_uint32_t VInputRingb;

		pnbuff = &pCtxt->SNbuff;
		/* In case of video chunk - hack, check for MJPEG header in the chunk, if not found then flag as partial frame */
		VidHdr = (pnbuff->pCurr[0]) | ((pnbuff->pCurr[1]) << 8);//need to check boundary of ring buffer

		/* hack for type of frame - whole frame or partial by checking the jpg header */
		if(FT_AVIP_JPGFRAMEHDR == VidHdr)
		{
			SMetaV.Flags |= (FT_AVIPMETAD_KEYFRAME | FT_AVIPMETAD_NEWFRAME);
			SMetaV.Pts = (paviv->CurrFrame*paviv->TimeScale*1000)/paviv->DataRate;//calculation of timestamp based on the fileformat - player needs to change this according to the playback timestamp
			SMetaV.Curr = pVFifo->fifo_wp;
			SMetaV.Length = VBuffSz;
			//printf("Video movi chk %x %x %x %d\n",movichk,chksz,VidHdr,SMetaV.Pts);
			/* reset the partial buffer flag of the previous chunk if any */
		}
		else
		{
			/* Partial frame, fetch the earlier queue element and set the partial frame flag */
			//printf("Video movi chk partial %x %x %x\n",movichk,chksz,VidHdr);
			FT_RQueue_PeekELatest(prqv,&SMetaV);
			SMetaV.Length += VBuffSz;
			SMetaV.Flags |= (FT_AVIPMETAD_PARTIALFRAME);
			SMetaV.Flags &= ((~FT_AVIPMETAD_NEWFRAME));//chunks other then first chunk of the frame
		}

		/* get the available data in FT81x */
		VInputRingb = Ft_Fifo_GetFreeSpace(pHHal,pVFifo);
		//VInputRingb = FT_AVIPARSER_TESTSTUB_VIDBUFFMAXSZ;//hack for fake sync

		/* Download data into FT81x */
		if(VInputRingb < VBuffSz)
		{
			/* again partial scenarios, so copy partial and re update the metadata element */

			VDatatocopy = VInputRingb;
			SMetaV.PartialSz += VDatatocopy;
			SMetaV.Flags |= FT_AVIPMETAD_PARTIALBUFFER;
			//printf("Video fstpar chk %x %x %x %x\n",movichk,chksz,VInputRingb,SMetaV.Flags);
		}
		else
		{
			VDatatocopy = VBuffSz;
			SMetaV.Flags &= (~FT_AVIPMETAD_PARTIALBUFFER);
		}
	}
	else
	{
		SMetaV.Length = 0;
		SMetaV.PartialSz = 0;
		SMetaV.Pts = ((paviv->CurrFrame*paviv->TimeScale*1000)/paviv->DataRate);//calculation of timestamp based on the fileformat - player needs to change this according to the playback timestamp
		SMetaV.Curr = pVFifo->fifo_wp;
		SMetaV.Flags = FT_AVIPMETAD_NEWFRAME;
	}
	//printf("bfrvfill %x %x\n",Ft_Gpu_Hal_Rd32(pvdp->phalctxt,REG_MEDIAFIFO_READ),Ft_Gpu_Hal_Rd32(pvdp->phalctxt,REG_MEDIAFIFO_WRITE));
	FT_AVIFillVdata(pCtxt,pvdp,VDatatocopy);
	//FT_AVIPFlush(pCtxt,VBuffSz);//fake sync
	if(SMetaV.Flags & (FT_AVIPMETAD_NEWFRAME))
	{
		FT_RQueue_WriteE(prqv,&SMetaV);
		//printf("Video meta %x %x %x %x %x %x %x %d %d %d\n",SMetaV.Length,SMetaV.PartialSz,SMetaV.Pts,SMetaV.Curr,SMetaV.Flags,Ft_Gpu_Hal_Rd32(pHHal,REG_MEDIAFIFO_READ),Ft_Gpu_Hal_Rd32(pHHal,REG_MEDIAFIFO_WRITE),prqv->ReadIdx,prqv->WriteIdx,prqv->ValidEle);
		paviv->CurrFrame++;
	}
	else
	{
		FT_RQueue_WriteEInplace(prqv,&SMetaV);
	}

	return FT_AVIPARSER_OK;
}

ft_int32_t FT_AVIPAudFillChunk(FT_AVIParser_t *pCtxt,FT_AudDataPath_t *padp,ft_uint32_t ABuffSz)
{
	ft_uint32_t AInputRingb,ADatatocopy = 0;
	FT_RingQueue_t *prqa;
	FT_RingBuffer_t *prba;
	FT_AviA_t *pavia;
	FT_MetaA_t SMetaA;


	/* as of now only video buffer size is loaded */
	prqa = padp->pairq;
	prba = padp->pairb;
	pavia = &pCtxt->SAud;

	FT_MetaA_Init(&SMetaA);
	/* Check for skip frames in case of video */
	if(pCtxt->Flags & (FT_AVIPARSER_DROPAUDIO | FT_AVIPARSER_DROPALLMEDIA | FT_AVIPARSER_FAKESYNC))
	{
		FT_AVIPFlush(pCtxt,ABuffSz);//fake sync
		FT_RQueue_WriteE(prqa,&SMetaA);//incorrect in case of partial frames present in chunk
		pavia->CurrFrame++;//incorrect in case of partial frames present in chunk
		//printf("Audiochk flush %x %x \n",ABuffSz,pCtxt->Flags);
		return FT_AVIPARSER_OK;//do not insert any metadata
	}

	if(ABuffSz > 0)
	{

		/* hack for pcm and mp3 assuming that chunk is independently decodable */
		SMetaA.Flags |= (FT_AVIPMETAD_KEYFRAME | FT_AVIPMETAD_NEWFRAME);
		SMetaA.Pts = ((pavia->CurrFrame*pavia->TimeScale*pavia->packet_size*1000)/pavia->DataRate);//calculation of timestamp based on the fileformat - player needs to change this according to the playback timestamp
		SMetaA.Curr = prba->Write;
		SMetaA.Length = ABuffSz;
		//printf("Audio movi chk %x %x %d\n",movichk,chksz,SMetaA.Pts);

		/* get the available data in FT81x */
		//AInputRingb = FT_RingBuffer_FreeSpace(prba);//hack for audio fake sync
		AInputRingb = FT_AVIPARSER_TESTSTUB_AUDBUFFMAXSZ;//hack for audio fake sync

		/* Download data into FT81x */
		if(AInputRingb < ABuffSz)
		{
			/* again partial scenarios, so copy partial and re update the metadata element */
			ADatatocopy = AInputRingb;
			SMetaA.PartialSz = AInputRingb;
			SMetaA.Flags |= FT_AVIPMETAD_PARTIALBUFFER;
		}
		else
		{
			ADatatocopy = ABuffSz;
			SMetaA.Flags &= (~FT_AVIPMETAD_PARTIALBUFFER);
		}
	}
	else
	{
		SMetaA.Length = 0;
		SMetaA.PartialSz = 0;
		//in case of pcm the packet_size is chunk size
		SMetaA.Pts = ((pavia->CurrFrame*pavia->TimeScale*pavia->packet_size*1000)/pavia->DataRate);//calculation of timestamp based on the fileformat - player needs to change this according to the playback timestamp
		SMetaA.Curr = prba->Write;
		SMetaA.Flags = FT_AVIPMETAD_NEWFRAME;
	}

	FT_AVIFillAdata(pCtxt, padp,ADatatocopy);
	//FT_AVIPFlush(pCtxt,ABuffSz);//fake sync
	if(SMetaA.Flags & (FT_AVIPMETAD_NEWFRAME))
	{
		//printf("Audio chk write %x %x %x %x\n",SMetaA.Length,SMetaA.PartialSz,ADatatocopy,SMetaA.Flags);
		FT_RQueue_WriteE(prqa,&SMetaA);
		pavia->CurrFrame++;
	}
	else
	{
		/* In case of partial buffer and partial frame - should not hit partial frame in audio case */
		FT_RQueue_WriteEInplace(prqa,&SMetaA);
	}

	return FT_AVIPARSER_OK;
}

//fill one chunk/frame for all the media - need to ensure 1 frame worth of data is available for each of the media
ft_int32_t FT_AVIParser_FillChunk(FT_AVIParser_t *pCtxt,FT_VidDataPath_t *pvdp,FT_AudDataPath_t *padp)
{
	FT_AVIPARSER_STATUS Status = FT_AVIPARSER_OK;
	ft_uint32_t movichk,chksz = 0,VDatatocopy;
	FT_RingQueue_t *prqa,*prqv;
	FT_RingBuffer_t *prba;
	FT_AviV_t *paviv;
	FT_AviA_t *pavia;
	

	/* Each time check for the end of movi chunk */
	/* Check if parser is in play state or avi header state */
	if((pCtxt->CurrStat.MoviOff >= pCtxt->Riff[pCtxt->CurrRiff].MoviSize))
	{
		//printf("riffmoving %x %x %x \n",pCtxt->CurrRiff,pCtxt->CurrStat.MoviOff,pCtxt->Riff[pCtxt->CurrRiff].MoviSize);
		/* Roll over of riff chunk */
		pCtxt->CurrRiff++;
		if(pCtxt->CurrRiff >= pCtxt->NumRiff)
		{
			printf("Riffeof %x %x %x \n",pCtxt->CurrRiff,pCtxt->CurrStat.MoviOff,pCtxt->Riff[pCtxt->CurrRiff].MoviSize);
			pCtxt->CurrRiff = pCtxt->NumRiff;//protection
			return FT_AVIPARSER_EOF;
		}
		/* Fetch new data into local buffer  */
		FT_AVIParser_SetRiff(pCtxt,pCtxt->CurrRiff);
	}
	
	/* as of now only video buffer size is loaded */	
	prqa = padp->pairq;
	prba = padp->pairb;
	pavia = &pCtxt->SAud;
	
	/* video specific local pointers */
	prqv = pvdp->pvirq;
	paviv = &pCtxt->SVid;
	
	/* Check for all the control flags of parser */
	/* Check for partial flag present in the current queue element */
	if((prqv->ValidEle >= prqv->TotEle) || (prqa->ValidEle >= prqa->TotEle))
	{
		//printf("parchk exceed valideles %d %d %d %d \n",prqv->ValidEle,prqv->TotEle,prqa->ValidEle,prqa->TotEle);
		/* Exceeded the limit */		
		return FT_AVIPARSER_BQEFULL;
	}

	Status = FT_AVIPVidChkFillParBuff(pCtxt,pvdp);
	if(Status == FT_AVIPARSER_PARTIALBUFFERUSECASE)
	{
		return FT_AVIPARSER_OK;
	}
	Status = FT_AVIPAudChkFillParBuff(pCtxt,padp);
	if(Status == FT_AVIPARSER_PARTIALBUFFERUSECASE)
	{
		return FT_AVIPARSER_OK;
	}
	
	/* Parse a new chunk */
	movichk = FT_AVIPReadL32(pCtxt);
	chksz = FT_AVIPReadL32(pCtxt);
	
	/* hack for any video convertor */
	if(chksz & 0x00000001)
	{
		//printf("Chunk is nonmultiple of 2 %x %x \n",movichk,chksz);
		chksz = chksz + 1;
	}

	/* Stats maintained by parser - make sure to take care movioff during seek implementation */
	pCtxt->CurrStat.MoviOff += chksz + 8;//two dwords from the above

	//printf("Parchk %x %x \n",movichk,chksz);
	/* Check for audio or video or text/midi chunks */
	if((movichk & 0xFFFFF0FF) == FT_AVI_4CC_00dc)
	{
		//timer_profilecountstart(&timercount,&timervalue);

		Status = FT_AVIPVidFillChunk(pCtxt,pvdp,chksz);

		//timer_profilecountend(&timercount,&timervalue);

	}
	else if((movichk & 0xFFFFF0FF) == FT_AVI_4CC_00wb)
	{
		Status = FT_AVIPAudFillChunk(pCtxt,padp,chksz);
	}
	else
	{
		printf("!AV chk %x %x\n",movichk,chksz);
		/* Flush this chunk and go for another chunk */
		Status = FT_AVIPFlush(pCtxt,chksz);
	}
			
	/* Copy data from parser to input audio buffer */	
	return Status;	
}


//flush all media contents
ft_int32_t	FT_AVIParser_FlushAll(FT_AVIParser_t *pCtxt);
//flush only specific media content
ft_int32_t	FT_AVIParser_Flush(FT_AVIParser_t *pCtxt);
//seek all the media contents to a specific time
ft_int32_t	FT_AVIParser_AviSeek(FT_AVIParser_t *pCtxt);
//free up every thing and exit
ft_int32_t	FT_AVIParser_Exit(FT_AVIParser_t *pCtxt)
{
	/* CLose any hanldes or context opened in avi parser init or steady state */
#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
	fclose(pCtxt->File);
#elif defined(FT900_PLATFORM)
	f_close(&pCtxt->File);
#endif
	pCtxt->Flags = 0;


	return FT_AVIPARSER_OK;
}



/* Nothing beyond this */
